package yrpg.gmscm.rc.rcd.agreement.list;

import cn.romens.cloud.xdepot.algo.RMAlgoQuery;
import cn.romens.cloud.xdepot.entity.base.RMEntityHelper;
import kd.bos.algo.DataSet;
import kd.bos.bill.BillShowParameter;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.list.events.BeforeShowBillFormEvent;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import yrpg.gmscm.rc.rcd.enumerate.AgmApplyTypeEnum;
import yrpg.gmscm.rc.rcd.enumerate.StatusEnum;
/**
 * @author zhangjiawei
 * 协议变更插件
 */
public class AgmApplyAlterationListPlugin extends AbstractListPlugin {
    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String operationKey = evt.getOperationKey();
        //协议变更
        if ("alteration".equals(operationKey)) {
            verifyBill(AgmApplyTypeEnum.CHANGE);
        }
        if ("stop_ags".equals(operationKey)) {
            verifyBill(AgmApplyTypeEnum.STOP);
        }
    }

    /**
     * 单据校验
     * @param type 类型
     */
    private void verifyBill(AgmApplyTypeEnum type){
        //校验是否只选中一条
        Object[] id = this.getSelectedRows().getPrimaryKeyValues();
        if (id.length != 1) {
            this.getView().showTipNotification("请选择一条协议进行"+type.getValue()+"操作！");
        } else {
            //是否已审核
            DynamicObject rmRcAgreementApply = BusinessDataServiceHelper.loadSingle(id[0], "rm_rc_agreement_apply");
            String status = rmRcAgreementApply.getString("billstatus");
            if (!StatusEnum.C.getKey().equals(status)) {
                this.getView().showTipNotification("单据未审核不允许"+type.getValue()+"！");
            } else {
                //是否已终止
                String rmAgreeNum = rmRcAgreementApply.getString("rm_agm_number");
                QFilter qFilter = new QFilter("rm_apply_type", QCP.equals, AgmApplyTypeEnum.STOP.getKey()).and("rm_agm_number", QCP.equals, rmAgreeNum);
                DynamicObject[] dynamicObject = BusinessDataServiceHelper.load("rm_rc_agreement_apply", "id", qFilter.toArray());
                if (dynamicObject.length != 0) {
                    this.getView().showTipNotification("协议已终止不允许进行变更操作！");
                } else {
                    verifySfExist(rmAgreeNum,rmRcAgreementApply.getPkValue(),type.getKey());
                }
            }
        }
    }


    /**
     * 触发复制功能 并传入类型值
     * @param rmAgreeNum 协议编码
     * @param pkValue 变更单据
     * @param type 类型
     */
    private void verifySfExist(String rmAgreeNum, Object pkValue, String type) {
        //查询未审核的变更协议
        QFilter qFilter = new QFilter("rm_agm_change_main", QCP.equals, rmAgreeNum).and("billstatus", QCP.not_equals, StatusEnum.C.getKey());
        QFilter qFilter1 = new QFilter("rm_agm_number", QCP.equals, rmAgreeNum).and("billstatus", QCP.not_equals, StatusEnum.C.getKey());
        QFilter qFilter2 = qFilter.or(qFilter1);
        DataSet queryDataSet =RMAlgoQuery.queryDataSet("rm_rc_agreement_apply", "billno", qFilter2.toArray(),null);
        if (!queryDataSet.isEmpty()) {
            this.getView().showTipNotification("协议变更失败! 未审核编号：" + queryDataSet.next().getString("billno"));
        } else {
            this.getView().getPageCache().put("change_bill",pkValue.toString());
            this.getView().getPageCache().put("change_type",type);
            this.getView().invokeOperation("copy_bill");
        }
    }


    @Override
    /*
     * 变更
     */
    public void beforeShowBill(BeforeShowBillFormEvent e) {
        super.beforeShowBill(e);
        String changeBill =this.getView().getPageCache().get("change_bill");
        String changeType =this.getView().getPageCache().get("change_type");
        if(changeBill!=null){
            //根据选中协议组合查询条件
            BillShowParameter billShowParameter= e.getParameter();
            Long pkId=(Long)billShowParameter.getPkId();
            DynamicObject load = RMEntityHelper.loadSingle("rm_rc_agreement_apply", pkId);
            DynamicObject oldData = RMEntityHelper.loadSingle("rm_rc_agreement_apply", changeBill);
            String agmChangeMain = load.getString("rm_agm_change_main");
            QFilter qFilter = new QFilter("rm_agm_change_main", QCP.equals, agmChangeMain);
            if (StatusEnum.A.getKey().equals(oldData.getString("rm_apply_type"))) {
                agmChangeMain = load.getString("billno");
                qFilter = new QFilter("rm_agm_change_main", QCP.equals, agmChangeMain);
                qFilter.or(new QFilter("billno", QCP.equals, agmChangeMain)) ;
            }
            //协议变更主单据编号

            if (!agmChangeMain.isEmpty()){
                //查询协议是否为最新单据
                DataSet dataSet = RMAlgoQuery.queryDataSet("rm_rc_agreement_apply", "id,auditdate,billno", new QFilter[]{qFilter},"auditdate desc");
                DataSet top = dataSet.top(1);
                DynamicObjectCollection dynamicObjects = ORM.create().toPlainDynamicObjectCollection(top);
                long oldChangeBillId = dynamicObjects.get(0).getLong("id");
                if(oldChangeBillId != 0L){
                    pkId=oldChangeBillId;
                }
                billShowParameter.setPkId(pkId);
                billShowParameter.getCustomParams().put("ischangebill",pkId);
                this.getView().getPageCache().remove("change_bill");
                billShowParameter.getCustomParams().put("change_type",changeType);
                this.getView().getPageCache().remove("change_type");

            }else {
                billShowParameter.getCustomParams().put("ischangebill",pkId);
                billShowParameter.getCustomParams().put("change_type",changeType);
            }
            //清楚缓存
            this.getView().getPageCache().remove("change_bill");
            this.getView().getPageCache().remove("change_type");
        }
    }
}
