package yrpg.gmscm.rc.rcd.agreement.operator;

import cn.romens.cloud.xdepot.algo.RMAlgoQuery;
import cn.romens.cloud.xdepot.plugin.op.RMAbstractOperationServicePlugIn;
import cn.romens.cloud.xfactory.exception.RMBizException;
import com.alibaba.druid.support.logging.Log;
import com.alibaba.druid.support.logging.LogFactory;
import com.alibaba.fastjson.JSONObject;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.db.tx.TX;
import kd.bos.db.tx.TXHandle;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.DeleteServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import yrpg.gmscm.rc.rcd.core.orm.IdGenerator;
import yrpg.gmscm.rc.rcd.core.orm.KDIdGenerator;
import yrpg.gmscm.rc.rcd.enumerate.AgmApplyTypeEnum;
import yrpg.gmscm.rc.rcd.enumerate.CalcmodeEnum;
import yrpg.gmscm.rc.rcd.enumerate.ClientTypeEnum;
import yrpg.gmscm.rc.rcd.tools.Const;
import yrpg.gmscm.rc.rcd.tools.GetFieldsHelper;
import yrpg.gmscm.rc.rcd.tools.SaveRecalculationLog;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author zhangjiawei
 * 返利协议拆分配置表
 */
public class AgmSplitConfigOpPlugin extends RMAbstractOperationServicePlugIn {
    Log log = LogFactory.getLog(AgmSplitConfigOpPlugin.class);
    IdGenerator idGenerator = new KDIdGenerator();

    //主信息映射返利协议
    private final  Map<String, String> rmRcConfigMainMapAgm = new HashMap<String, String>(){{
        //客商品明细id
        put("rm_rc_policyrecord_dtl", "rm_rc_policyrecord_dtl.id");
        //子明细id
        put("rm_rc_policy_dtl_dtl", "rm_rc_policyrecord_dtl.rm_rc_policy_dtl_dtl.id");
        //条款id
        put("rm_protocolid", "id");
        //条款名称
        put("rm_protocol_name", "rm_name");
        //核算组织
        put("rm_business", "org");
        //销售组织
        put("rm_marketorg", "rm_saleorg");
        //返利类型
        put("rm_rc_type", "rm_rc_type");
        //返利模式类型
        put("rm_rc_mode_type", "rm_rcmodel_type");
        //计算方式
        put("rm_calc_mode", "rm_calcmode");
        //基准日期
        put("rm_datum_date", "rm_standard_date");
        //计算基准
        put("rm_calc_datum", "rm_calc_standard");
        //返利计算规则
        put("rm_calculate_rule", "rm_rc_calcrule");
        //协议年度
        put("rm_policy_year", "rm_agreement_year");
        //客户维度
        put("rm_customer_type", "rm_cus_latitude");
    }};
    private final  Map<String, String> rmRcConfigMainMapOther = new HashMap<String, String>(){{
        //计算基准字段
        put("rm_calc_datum_field", "rm_field");
        //业务清单排序基准字段
        put("rm_biz_order_field", "rm_rc_config_order_field");
        //公式
        put("rm_formula", "rm_formula");
        //基准日期字段
        put("rm_datum_date_field", "rm_datum_date_field");
        //业务清单
        put("rm_rc_biz_name", "rm_rc_config_biz_info_number");

    }};
    //挡位信息映射
    private final  Map<String, String> rmRcConfigGearMapAgm = new HashMap<String, String>(){{
        //开始值
        put("rm_begin_value", "rm_rc_policyrecord_dtl.rm_rc_policy_dtl_dtl.rm_begin");
        //结束值
        put("rm_end_value", "rm_rc_policyrecord_dtl.rm_rc_policy_dtl_dtl.rm_end");
        //返利系数
        put("rm_rc_coefficient", "rm_rc_policyrecord_dtl.rm_rc_policy_dtl_dtl.rm_rc_coefficient");
        //起止日期
        put("rm_start_date", "rm_rc_policyrecord_dtl.rm_rc_policy_dtl_dtl.rm_date_begin");
        put("rm_end_date", "rm_rc_policyrecord_dtl.rm_rc_policy_dtl_dtl.rm_date_end");
        //返利协议子明细id
        put("rm_source_bill_entity", "rm_rc_policyrecord_dtl.rm_rc_policy_dtl_dtl.id");
        //客户维度
        put("rm_customer_type", "rm_cus_latitude");
        //协议编号
        put("rm_protocol_no", "rm_agm_number");

    }};
    //物料信息映射
    private final  Map<String, String> rmRcConfigGoodsMapAgm = new HashMap<String, String>(){{
        //限制值id
        put("rm_limit_value", "rm_rc_policyrecord_dtl.rm_material.id");
        //限制值编码
        put("rm_limit_code", "rm_rc_policyrecord_dtl.rm_material.number");
        //协议编码
        put("rm_policyrecord_billno", "rm_agm_number");
        //挡位id
        put("rm_gear_id_no", "rm_rc_policyrecord_dtl.id");
    }};
    //协议重算信息映射
    private final  Map<String, String> rmRcProtocolChangedMapAgm = new HashMap<String, String>(){{
        //返利类型
        put("rm_rc_type", "rm_rc_type");
        //销售组织
        put("rm_marketorg", "rm_saleorg");
        //核算组织
        put("rm_business", "rm_accorg");
    }};
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("rm_calcmode");
        fieldKeys.add("rm_calcmode.number");
        fieldKeys.add("rm_material_bundle");
        fieldKeys.add("rm_rc_calcrule");
        fieldKeys.add("rm_isgift");
        fieldKeys.add("rm_rc_policyrecord_gift");
        fieldKeys.add("rm_gift_invprice");
        fieldKeys.add("rm_rc_policyrecord_gift.rm_gift_invprice");
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        TXHandle tx = TX.required();
        try {
            DynamicObject[] dynamicObjects = e.getDataEntities();
            Set<Object> fromRuleIdList = new HashSet<>();
            Set<String> deleteConfigBillNo = new HashSet<>();
            for (DynamicObject dynamicObject : dynamicObjects) {
                DynamicObject rmCalcMode = dynamicObject.getDynamicObject("rm_calcmode");
                String number = rmCalcMode.getString("number");
                //计算方式为常规
                if (Objects.equals(CalcmodeEnum.COMMON.getKey(), number)) {
                    fromRuleIdList.add(dynamicObject.getPkValue());
                }
                String rmApplyType = dynamicObject.getString("rm_apply_type");
                //协议变更时要删除原来的数据 重新生成
                if (!rmApplyType.equals(AgmApplyTypeEnum.NEW.getKey())) {
                    deleteConfigBillNo.add(dynamicObject.getString("rm_agm_number"));
                }

            }
            //删除原数据，重新生成（变更时）
            deleteConfigInfo(deleteConfigBillNo);
            if (fromRuleIdList.size()!=0) {
                //常规-主信息-挡位-物料
                splitAgmRule(fromRuleIdList);
            }

            tx.commit();
        } catch (Exception exception) {
            tx.markRollback();
            throw new RMBizException(getStackTraceAsString(exception));
        } finally {
            tx.close();
        }

    }
    private void deleteConfigInfo(Set<String> deleteConfigBillNo){
        QFilter deleteFilter = new QFilter("rm_policyrecord_number",QCP.in,deleteConfigBillNo);
        //删除 条款主信息
        DeleteServiceHelper.delete("rm_rc_protocol_main",deleteFilter.toArray());
        //删除 协议条款档位
        deleteFilter = new QFilter("rm_protocol_no",QCP.in,deleteConfigBillNo);
        DeleteServiceHelper.delete("rm_rc_protocol_gear",deleteFilter.toArray());
        //删除 协议商品限制
        deleteFilter = new QFilter("rm_policyrecord_billno",QCP.in,deleteConfigBillNo);
        DeleteServiceHelper.delete("rm_rc_protocol_goods",deleteFilter.toArray());

    }



    /**
     * 生成源dataset
     *
     * @param fields 字段
     * @param idList 条件
     * @return dataset
     */
    private DataSet makeMain(String fields, Set<Object> idList) {
        //返利协议
        DataSet dataSet = RMAlgoQuery.queryDataSet("rm_rc_agreement_apply",
                fields+",rm_client,rm_client_group",
                new QFilter[]{new QFilter("id", QCP.in, idList)},
                null);
        //业务清单与计算基准关系
        DataSet rmRcConfigBizCalc = RMAlgoQuery.queryDataSet("rm_rc_config_biz_calc",
                "rm_rc_calcu_basis,rm_field",
                new QFilter[]{},
                null);
        dataSet = dataSet
                .leftJoin(rmRcConfigBizCalc)
                .on("rm_calc_standard", "rm_rc_calcu_basis")
                .select(dataSet.getRowMeta().getFieldNames(), new String[]{"rm_field"})
                .finish();
        //取基准日期
        DataSet rmRcRebateType = RMAlgoQuery.queryDataSet("rm_rc_rebate_type",
                "id,rm_rc_config_biz_info,rm_rc_config_biz_info.number as rm_rc_config_biz_info_number",
                new QFilter[]{},
                null);

        dataSet = dataSet
                .leftJoin(rmRcRebateType)
                .on("rm_rc_type", "id")
                .select(dataSet.getRowMeta().getFieldNames(), new String[]{"rm_rc_config_biz_info","rm_rc_config_biz_info_number"})
                .finish();
        DataSet rmRcConfigDatumDate = RMAlgoQuery.queryDataSet("rm_rc_config_datum_date",
                "rm_datum_date,rm_rc_config_biz_info,rm_field as rm_datum_date_field",
                new QFilter[]{},
                null);
        dataSet = dataSet
                .leftJoin(rmRcConfigDatumDate)
                .on("rm_rc_config_biz_info", "rm_rc_config_biz_info")
                .on("rm_standard_date", "rm_datum_date")
                .select(dataSet.getRowMeta().getFieldNames(), new String[]{"rm_datum_date_field"})
                .finish();
        //返利计算规则
        DataSet rmRcCalculateRule = RMAlgoQuery.queryDataSet("rm_rc_calculate_rule",
                "id,rm_formula,rm_rc_config_order,rm_rc_config_order_field",
                new QFilter[]{},
                null);
        dataSet = dataSet
                .leftJoin(rmRcCalculateRule)
                .on("rm_rc_calcrule", "id")
                .select(dataSet.getRowMeta().getFieldNames(), new String[]{"rm_rc_config_order_field", "rm_formula"})
                .finish();
        dataSet = dataSet.addField(
                "case" +
                        " when rm_cus_latitude = '"+ ClientTypeEnum.ONE.getKey() +"' then rm_client" +
                        " when rm_cus_latitude = '"+ ClientTypeEnum.GROUP.getKey() +"' then rm_client_group" +
                        " else 0 end",
                "rm_ety_customer_type");
        return dataSet;
    }

    //常规的保存
    private void splitAgmRule(Set<Object> idList) {
        List<DynamicObject> newConfigMainList = new ArrayList<>();
        List<DynamicObject> newConfigGearList = new ArrayList<>();
        List<DynamicObject> newConfigGoodsList = new ArrayList<>();
        List<DynamicObject> newPolicyRecordList = new ArrayList<>();

        HashSet<String> hashSet = new HashSet<>(rmRcConfigMainMapAgm.values());
        hashSet.addAll(rmRcConfigGearMapAgm.values());
        hashSet.addAll(rmRcConfigGoodsMapAgm.values());
        String valuesString = hashSet.stream()
                .map(Object::toString)
                .collect(Collectors.joining(","));
        DataSet dataSet = makeMain(valuesString, idList);

        Map<String, String> hashMap = new HashMap<>(rmRcConfigMainMapAgm);
        int sequence = 1;
        for (Row row : dataSet) {
            //主信息
            DynamicObject newConfigMain = assembleConfigMain(row, hashMap);
            String billno = newConfigMain.getString("billno");
            //挡位
            DynamicObject newConfigGear = assembleConfigGear(row, sequence, billno);
            //物料
            DynamicObject newConfigGoods = assembleGoods(row, billno);
            //重算
            DynamicObject newPolicyRecord = saveAgm(row, billno);

            newPolicyRecordList.add(newPolicyRecord);
            newConfigMainList.add(newConfigMain);
            newConfigGearList.add(newConfigGear);
            newConfigGoodsList.add(newConfigGoods);
            //分批保存
            divideSave(newConfigMainList, 2000);
            divideSave(newConfigGearList, 2000);
            divideSave(newConfigGoodsList, 2000);
            divideSave(newPolicyRecordList, 2000);
            sequence++;
        }
        divideSave(newConfigMainList, 1);
        divideSave(newConfigGearList, 1);
        divideSave(newConfigGoodsList, 1);
        divideSave(newPolicyRecordList, 1);

    }

    //保存协议重算
    public  DynamicObject saveAgm(Row row, String calcBillNo) {
        JSONObject jsonObject = new JSONObject();
        for (Map.Entry<String, String> entry : rmRcProtocolChangedMapAgm.entrySet()) {
            String key = entry.getKey();
            String value = entry.getValue();
            if (rmRcConfigMainMapAgm.containsValue(value)) {
                jsonObject.put(key, row.get(value));
            }
        }
        jsonObject.put("rm_protocol_no",row.getString("rm_agm_number"));
        return SaveRecalculationLog.saveProtocolChanged(jsonObject, calcBillNo,"返利协议-新增");
    }

    //组装主信息数据
    private DynamicObject assembleConfigMain(Row row, Map<String, String> policyRecordMap) {
        DynamicObject newConfigMain = BusinessDataServiceHelper.newDynamicObject("rm_rc_config_main");
        //返利协议编码
        newConfigMain.set("rm_policyrecord_number", row.get("rm_agm_number"));
        //单据编码
        String billNo = GetFieldsHelper.getBillNo(newConfigMain);
        newConfigMain.set("billno", billNo);
        //单据状态
        newConfigMain.set("billstatus", "C");
        //版本
        newConfigMain.set("rm_version", 1);
        //是否指定商品
        newConfigMain.set("rm_is_assign_goods", true);
        //返利条款分类
        newConfigMain.set("rm_agreement_type", "B");
//        if (policyRecordMap.containsValue("rm_iscus_reach") || rmRcConfigMainMapOther.containsValue("rmRcConfigMainMapOther")) {
//            if (!row.getBoolean("rm_iscus_reach")) {
//                //模型划分维度
//                newConfigMain.set("rm_model_dimension", "rm_terminalid");
//            }
//        }
        //返利协议赋值
        policyRecordMap
                .keySet()
                .forEach(key -> newConfigMain.set(key, row.get(policyRecordMap.get(key))));
        //其他关联的赋值
        rmRcConfigMainMapOther
                .keySet()
                .forEach(key -> newConfigMain.set(key, row.get(rmRcConfigMainMapOther.get(key))));
        return newConfigMain;
    }

    //组装挡位信息数据
    private DynamicObject assembleConfigGear(Row row, int sequence, String billNo) {
        DynamicObject newConfigGear = BusinessDataServiceHelper.newDynamicObject("rm_rc_config_gear");
        //条款编码
        newConfigGear.set("billno", billNo);
        //条款编码-计算用
        newConfigGear.set("rm_protocol_calc_no", billNo);
        //单据状态
        newConfigGear.set("billstatus", "C");
        //排序
        newConfigGear.set("rm_order", sequence);
        //版本
        newConfigGear.set("rm_version", 1);
        //客户维度值
        newConfigGear.set("rm_ety_customer_type", row.get("rm_ety_customer_type"));

        rmRcConfigGearMapAgm
                .keySet()
                .forEach(key -> newConfigGear.set(
                        key,
                        row.get(rmRcConfigGearMapAgm.get(key))
                ));
        if (newConfigGear.getBigDecimal("rm_begin_value").compareTo(newConfigGear.getBigDecimal("rm_end_value")) == 0) {
            newConfigGear.set("rm_begin_value", newConfigGear.getBigDecimal("rm_end_value"));
            newConfigGear.set("rm_end_value", Const.MAX_GEAR_NUM);
        }
        return newConfigGear;
    }

    //组装物料信息
    private DynamicObject assembleGoods(Row row, String billno) {
        DynamicObject newConfigGoods = BusinessDataServiceHelper.newDynamicObject("rm_rc_config_goods");
        //单据状态
        newConfigGoods.set("billstatus", "C");
        //商品限制类型
        newConfigGoods.set("rm_goods_type", "A");
        //版本
        newConfigGoods.set("rm_version", 1);
        //条款编码
        newConfigGoods.set("billno", billno);
        rmRcConfigGoodsMapAgm
                .keySet()
                .forEach(key -> newConfigGoods.set(key, row.get(rmRcConfigGoodsMapAgm.get(key))));
        return newConfigGoods;
    }

    /**
     * 分批保存
     *
     * @param arrayList 保存的数据
     * @param n         n条保存一次
     */
    private void divideSave(List<DynamicObject> arrayList, int n) {
        if (arrayList.size() >= n) {
            SaveServiceHelper.save(arrayList.toArray(new DynamicObject[0]));
            arrayList.clear();
        }
    }
    /**
     * 详细错误信息
     * @param throwable 异常
     * @return 详细错误信息
     */
    public static String getStackTraceAsString(Throwable throwable) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        throwable.printStackTrace(pw);
        return sw.toString();
    }
}
