package yrpg.gmscm.rc.rcd.apply.form;

import cn.romens.cloud.sdk.entity.FormEntryFastWriter;
import cn.romens.cloud.xdepot.algo.RMAlgoQuery;
import cn.romens.cloud.xdepot.entity.base.RMEntityHelper;
import cn.romens.cloud.xdepot.plugin.form.RMAbstractFormPlugin;
import com.alibaba.druid.util.StringUtils;
import com.alibaba.fastjson.JSONObject;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.algo.RowMeta;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.entity.MainEntityType;
import kd.bos.form.IFormView;
import kd.bos.form.container.Container;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.operate.FormOperate;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import yrpg.gmscm.rc.rcd.tools.GetDateUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.EventObject;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;


/**
 * @author zhangjiawei
 * @date 2023/8/9 13:51
 * 下游返利向导查询数据插件
 */
public class WizardQueryFormPlugin extends RMAbstractFormPlugin {
    /**
     * 向导表字段对照结果表字段
     */
    private final Map<String, String> wizardFieldsMap = new HashMap<String, String>() {{
        //put(向导表，结果表)
        put("rm_rc_type", "rm_rc_type");//返利类型
        put("rm_qty", "rm_qty");//数量
        put("rm_unit_price", "rm_unit_price");//返利系数
        put("rm_biz_date", "rm_biz_date");//业务日期
        put("rm_account_org", "rm_account_org");//核算组织
        put("rm_sale_org", "rm_sale_org");//核算组织
        put("rm_cope_rc", "rm_cope_rc");//应付返利
        put("rm_src_bill_rowid", "entity");//结果表业务清单行id
        put("rm_src_billid", "id");//结果表id
        put("rm_src_bill_no", "billno");//结果表编码
        put("rm_the_apply_rc", "");//本次申请返利

        //
    }};

    /**
     * 查询触发
     */
    @Override
    public void afterDoOperation(AfterDoOperationEventArgs args) {
        FormOperate formOperate = (FormOperate) args.getSource();
        if (StringUtils.equals("query", formOperate.getOperateKey())) {
            if (args.getOperationResult().isSuccess()) {
                //校验必填项
                Object rmDateBegin = this.getModel().getValue("rm_date_begin");
                Object rmDateEnd = this.getModel().getValue("rm_date_end");
                Object rmFilterSaleOrg = this.getModel().getValue("rm_filter_sale_org");
                if (rmDateBegin == null || rmDateEnd == null) {
                    this.getView().showTipNotification("开始时间或结束时间未填写！");
                } else if (rmFilterSaleOrg == null) {
                    this.getView().showTipNotification("请检查必录项！");
                } else {
                    //情况原单据体数据
                    DynamicObjectCollection entity = this.getModel().getEntryEntity("rm_rcd_wizard_entry");
                    entity.clear();
                    writeImportForm();
                }

            }
        }
    }

    @Override
    public void afterBindData(EventObject e) {
        Container rmScreeningCondition = this.getView().getControl("rm_screening_condition");
        rmScreeningCondition.setCollapse(false);
    }

    /**
     * 组装过滤条件
     *
     * @return 过滤条件
     */
    private QFilter getFilter() {
        Date begin = (Date) this.getModel().getValue("rm_date_begin");//开始值
        Date end = (Date) this.getModel().getValue("rm_date_end");//结束值
        DynamicObject saleSection = (DynamicObject) this.getModel().getValue("rm_filter_sale_org");//销售组织
        DynamicObject rmMaterial = (DynamicObject) this.getModel().getValue("rm_filter_material");//物料
        DynamicObject rmFilterType = (DynamicObject) this.getModel().getValue("rm_filter_type");//返利类型
        String rmFilterSourceBillno = (String) this.getModel().getValue("rm_filter_source_billno");//结果表编号
        boolean rmFilterIfArtificial = (Boolean) this.getModel().getValue("rm_filter_if_artificial");//手工录入
        QFilter qFilter = new QFilter("id", QCP.not_equals, 0);
        if (end != null) {//结束值不为空
            end = GetDateUtils.toDayEnd(end);
            qFilter = qFilter.and("rm_rcd_rc_result_biz.rm_biz_date",
                    QCP.less_equals, end);
        }
        if (begin != null) {//开始值不为空
            begin = GetDateUtils.toDayStart(begin);
            qFilter = qFilter.and("rm_rcd_rc_result_biz.rm_biz_date",
                    QCP.large_equals, begin);
        }
        if (saleSection != null) {//销售组织
            qFilter = qFilter.and("rm_rcd_rc_result_biz.rm_sale_org",
                    QCP.equals, saleSection.getPkValue());
        }
        if (rmMaterial != null) {//物料
            qFilter = qFilter.and("rm_rcd_rc_result_biz.rm_material",
                    QCP.equals, rmMaterial.getPkValue());
        }
        if (rmFilterType != null) {//返利类型
            qFilter = qFilter.and("rm_rc_type.id",
                    QCP.equals, rmFilterType.getPkValue());
        }
        if (rmFilterSourceBillno != null && !"".equals(rmFilterSourceBillno)) {//结果表编号（测试使用）
            qFilter = qFilter.and("billno", QCP.equals, rmFilterSourceBillno);
        }
        if (!rmFilterIfArtificial) {
            qFilter = qFilter.and("rm_if_artificial", QCP.equals, rmFilterIfArtificial);
        }
        return qFilter;
    }

    /**
     * 需要查询的字段
     */
    private String getQueryFields(List<String> fields) {
        MainEntityType billType = RMEntityHelper.getEntityType("rm_rcd_rc_result");
        Map<String, IDataEntityProperty> allFields = billType.getAllFields();
        Set<String> resultSet = new HashSet<>();
        for (String field : allFields.keySet()) {
            if (!fields.contains(field)) {
                continue;
            }
            if (allFields.get(field).getParent() != null) {
                String entity = allFields.get(field).getParent().getName();
                if (!"rm_rcd_rc_result".equals(entity)) {
                    resultSet.add(entity + "." + field + " as " + field);
                    resultSet.add(entity + ".id as entity");

                } else {
                    resultSet.add(field);
                }
            }
        }
        resultSet.add("id");
        return String.join(",", resultSet);
    }

    /**
     * 获取结果表的查询数据
     */
    private DataSet getDataSet() {
        List<String> list = new ArrayList<>(wizardFieldsMap.values());
        String rmRcdCalcResult = getQueryFields(list);
        //获取过滤条件
        QFilter qFilter = getFilter();
        DataSet dataSet = RMAlgoQuery.queryDataSet("rm_rcd_rc_result",
                rmRcdCalcResult,
                qFilter.toArray(),
                null);
        //条件判断 未查询到数据
        if (dataSet.isEmpty()) {
            this.getView().showTipNotification("查询结果为空！");
            return null;
        }
        //申请日志表的查询
        String qFilterIdField = "rm_rcd_rc_result_record.rm_business_list";
        String queryApplyFields = "rm_rcd_rc_result_record.rm_business_list as rm_business_list ," +
                "rm_rcd_rc_result_record.rm_type as rm_type," +
                "rm_rcd_rc_result_record.id as rm_rcd_rc_result_record_id," +
                "rm_rcd_rc_result_record.rm_the_apply_rc as rm_the_apply_rc";
        DataSet dataSetApplyLog = getResultLogData(dataSet.copy(), qFilterIdField, queryApplyFields);
        dataSetApplyLog = dataSetApplyLog
                .updateField("rm_the_apply_rc",
                        "case when rm_type = 'B' then  rm_the_apply_rc * -1 else rm_the_apply_rc end");
        dataSetApplyLog = dataSetApplyLog
                .groupBy(new String[]{"rm_business_list"})
                .sum("rm_the_apply_rc")
                .finish();


        RowMeta rowMeta = dataSet.getRowMeta();
        dataSet = dataSet
                .leftJoin(dataSetApplyLog)
                .on("entity", "rm_business_list")
                .select(rowMeta.getFieldNames(), new String[]{"rm_business_list", "rm_the_apply_rc"})
                .finish();
        dataSet = dataSet.updateField("rm_the_apply_rc",
                "case when rm_the_apply_rc = null then 0 else rm_the_apply_rc end  as rm_the_apply_rc");
        dataSet = dataSet.orderBy(new String[]{"rm_biz_date"});
        return dataSet;
    }

    /**
     * 查询结果表的日志表
     *
     * @param dataSet          源数据
     * @param qFilterIdField   日志表的业务清单id字段
     * @param queryApplyFields 日志表需要查询的字段
     * @return 日志表的 dataset
     */
    private DataSet getResultLogData(DataSet dataSet, String qFilterIdField, String queryApplyFields) {
        HashSet<Long> entity = new HashSet<>();
        List<QFilter> qFilterList = new ArrayList<>();
        for (Row row : dataSet) {
            entity.add(row.getLong("entity"));
            if (entity.size() == 400000) {
                QFilter qFilter1 = new QFilter(qFilterIdField, QCP.in, entity);
                qFilterList.add(qFilter1);
                entity.clear();
            }
        }
        if (entity.size() != 0) {
            QFilter qFilter1 = new QFilter(qFilterIdField, QCP.in, entity);
            qFilterList.add(qFilter1);
        }
        QFilter qFilterInit = new QFilter("id", QCP.equals, 0);
        DataSet applyLog = RMAlgoQuery
                .queryDataSet("rm_rcd_rc_result",
                        queryApplyFields,
                        qFilterInit.toArray(),
                        null);
        for (QFilter filter : qFilterList) {
            DataSet dataSetApplyLogInit = RMAlgoQuery
                    .queryDataSet("rm_rcd_rc_result",
                            queryApplyFields,
                            filter.toArray(),
                            null); //查询
            applyLog = applyLog.union(dataSetApplyLogInit);
        }
        return applyLog;
    }

    /**
     * 查询结果表并写入
     */
    private void writeImportForm() {
        DataSet dataSet = getDataSet();
        if (dataSet != null) {
            //写入向导弹窗
            IFormView view = this.getView();
            FormEntryFastWriter entryFastWriter = FormEntryFastWriter.create(view, "rm_rcd_wizard_entry");
            int findCount = 0;
            for (Row row : dataSet) {
                if ((row.getBigDecimal("rm_the_apply_rc").compareTo(row.getBigDecimal("rm_cope_rc")) == 0)) {
                    continue;
                }
                JSONObject jsonObject = newCreateRow(row);
                entryFastWriter.addRow(jsonObject);
                findCount++;
            }
            if (findCount == 0) {
                this.getView().showTipNotification("查询结果为空！");
                return;
            }
            entryFastWriter.batchCreateNewEntryRow();
            entryFastWriter.updateView();
            Container rmScreeningCondition = this.getView().getControl("rm_screening_condition");
            rmScreeningCondition.setCollapse(true);
        }

    }

    /**
     * 处理单据体数据
     *
     * @param data 源
     * @return json
     */
    private JSONObject newCreateRow(Row data) {
        //已申请返利
        BigDecimal aftApplyRc = data.getBigDecimal("rm_the_apply_rc");
        //应付返利
        BigDecimal rmCopeRc = data.getBigDecimal("rm_cope_rc");
        //应付返利不能为空 为空给默认赋值0
        if (rmCopeRc == null) {
            rmCopeRc = BigDecimal.ZERO;
        }
        //未申请返利
        BigDecimal unAftApplyRc = rmCopeRc.subtract(aftApplyRc);
        JSONObject rowJson = new JSONObject();
        for (String field : wizardFieldsMap.keySet()) {
            if ("rm_unaftapplyrc".equals(field)) {
                rowJson.put(field, unAftApplyRc);
                continue;
            }
            if ("rm_the_apply_rc".equals(field)) {
                rowJson.put(field, unAftApplyRc);
                continue;
            }
            rowJson.put(field, data.get(wizardFieldsMap.get(field)));
        }
        return rowJson;
    }


}
